unit Graphe;

interface

uses windows, SysUtils, Graphics, Classes,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs, inifiles,
   maths, gtsUtil, uniteKer, gtsdef, constgts, Math;

const
   IntSansSignif     = -1;
   dimPoint: integer = 2;
   scaleCurseur      = 10;
   LongNombreAxe     = 4;
   NbreGraduations   = 10;
   NbreGradMin       = 5;
   Fleche: array[boolean] of char = (#192, #193);

type
   TCourbe = class(Tvoie)
   private
      a, b: double; { entier = rel*a+b }
   public
      deltaAxe, exposant, pointDebut: double;
      { deltaAxe = cart entre graduations sur les axes }
      axeLog, axeDateTime: boolean;
      NbreChiffre: byte;
      courbeBottom, courbeTop: integer;
      LabelVoie: Tlabel;
      GBvoie: TgroupBox;
      constructor Create(Anom: string; Aunite: string;
         Asignif: string; Anumero: integer);
      destructor Destroy; override;
      procedure setDelta(grandAxe: boolean);
      function FormatAxe(x: double): string;
      function XWindow(r: double): integer;
      function XMonde(xi: integer): double;
   end;

   TvoieNonEA = class(Tcourbe)
      procedure setMinMax; override;
   end;

   Tmagnum = class(TvoieNonEA)
   private
      compteurDebut: longint;
      FmodeAngulaire, FmodeRadian: boolean;
      procedure setModeAngulaire(Value: boolean);
      procedure setModeRadian(Value: boolean);
   public
      increment:    double;
      TraitMagnumE: double; // E=effective
      NbreMax:      integer;
      Negatif:      boolean;
      vecteurLu:    vecteurInt;
      reponse:      ansiString;
      NbreMagnum:   integer;
      isChute:      boolean;
      property modeAngulaire: boolean Read FmodeAngulaire Write setModeAngulaire;
      property modeRadian: boolean Read FmodeRadian Write setModeRadian;
      constructor Create;
      function affVoltmetre: string; override;
      function calibreVoie: string; override;
      function Date: double;
      function valeurReal(V: integer): double; override;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
      procedure testChute;
      procedure recopieMagnum;
   end;

   TsortieAnalogique = class(TvoieNonEA)
   protected
      function GetNomVoieOrphy: string; override;
   public
      DebutSortie: integer;
      FinSortie:   integer;
      DeltatSA:    integer; { ms }
      NbreInterSA: integer;
      Inverse:     boolean;
      constructor Create;
      procedure calcVecteur;
      function valeurVolt(V: integer): double; override;
      function calibreVoie: string; override;
      function valeurReal(V: integer): double; override;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
   end;

   TentreeClavier = class(TvoieNonEA)
      function GetNomVoieOrphy: string; override;
      constructor Create;
   end;

   TbaseDeTemps = class(TvoieNonEA)
   public
      Deltat:   extended;
      pea:      Tpea;
      ModeLent: boolean;
      FreqMin, freqMax: boolean;
      constructor Create;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
   end;

   TgrapheGTS = class
   private
      FNbreInter: integer;
      procedure setNbreInter(Avalue: integer);
   public
      Canvas:        Tcanvas;
      oTraceLigne:   boolean;
      oTracePoints:  boolean;
      NbreCourbe:    integer;
      codeErreur:    integer;
      avecAxeX, avecAxeY, avecUnAxe: boolean;
      grandAxeX, grandAxeY: boolean;
      voieOrphy:     array[indiceVoie] of Tcourbe;
      LimiteCourbe, LimiteFenetre: Trect;
      LargCarac, HautCarac, Marge: integer;
      NbreVoie, NbrePea: integer;
      NbreAff:       integer;
      voieAbscisse:  indiceVoie;
      quadrillage, AnalyseurLogique: boolean;
      ModeAcq:       TModeAcq;
      ReticuleTrace: boolean;
      ModeSynchro:   TModeSynchro;
      voieSynchro:   indiceVoie;
      SynchroUp:     boolean;
      frontSynchro:  integer;
      seuilSynchro:  integer;
      deltaBouge:    integer;
      yseuil:        integer;
      property NbreInter: integer Read FnbreInter Write setNbreInter;
      constructor Create;
      procedure Draw;
      procedure TraceCourbes;
      procedure TracePoints(jdebut,jfin : integer);
      procedure TraceUnPoint(voie: integer);
      procedure TraceReticule(voie: integer);
      procedure SetHauteurFonte(H: integer);
      procedure TraceLigneSynchro;
      destructor Destroy; override;
      procedure TraceMotif(xi, yi: integer; motif: Tmotif);
   end;

   TentreeBinaire = class(Tcourbe)
   private
      FnbreEB: byte;
      Masque:  word;
      procedure setNbreEB(Value: byte);
   protected
      function GetNomVoieOrphy: string; override;
   public
      constructor Create;
      function calibreVoie: string; override;
      property NbreEB: byte Read FnbreEB Write setNbreEB;
      function valeurReal(V: integer): double; override;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
      function NomValeurUnite(nombre: double): string; override;
   end;

var
   grapheGts:        TgrapheGTS;
   sortieAnalogique: TsortieAnalogique;
   tension:          Tunite;
   BaseDeTemps:      TbaseDeTemps;
   EntreeClavier:    TentreeClavier;
   EntreeBinaire:    TentreeBinaire;
   Magnum:           Tmagnum;
   Frequence:        TvoieNonEA;

implementation

var
   MaxGr, MinGr: integer;

function Tcourbe.XMonde(xi: integer): double;
begin
   Result := (xi - B) / A;
end;

function Tcourbe.XWindow(r: double): integer;
begin
   if isnan(r) then
      Result := IntSansSignif
   else if AxeLog then
      if r > 0 then begin
         try
            Result := round(A * Log10(r) + B)
         except
            Result := IntSansSignif
         end;
      end
      else
         Result := IntSansSignif
   else
      Result := round(A * r + B);
end;

procedure TgrapheGTS.TraceMotif(xi, yi: integer; motif: Tmotif);

   procedure Segment(x1, y1, x2, y2: integer);
   begin
      Canvas.moveTo(x1, y1);
      Canvas.lineTo(x2, y2);
   end;

var
   PolyLigne: array[1..5] of Tpoint;
begin
   case motif of
      mCroix: begin
         Segment(xi - dimPoint, yi, xi + dimPoint, yi);
         Segment(xi, yi - dimPoint, xi, yi + dimPoint);
      end;
      mCroixDiag: begin
         segment(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint);
         segment(xi + dimPoint, yi - dimPoint, xi - dimPoint, yi + dimPoint);
      end;
      mCercle: canvas.ellipse(xi - dimPoint, yi - dimPoint, xi +
            dimPoint + 1, yi + dimPoint + 1);
      mCarre: canvas.rectangle(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint);
      mLosange: begin
         PolyLigne[1] := Point(xi - dimPoint, yi);
         PolyLigne[2] := Point(xi, yi - dimPoint);
         PolyLigne[3] := Point(xi + dimPoint, yi);
         PolyLigne[4] := Point(xi, yi + dimPoint);
         PolyLigne[5] := Point(xi - dimPoint, yi);
         canvas.polyLine(PolyLigne);
      end;
   end;
end;

procedure TgrapheGTS.TraceUnPoint(voie: integer);
var
   xi, yi: integer;
begin with voieOrphy[voie] do begin
      canvas.pen.color := couleur;
      canvas.brush.color := couleur;
      if (modeAcq in [AcqClavier, AcqPoint]) and (NbreAff < 32) then
         canvas.pen.Width := 3;
      xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].valeurAff);
      yi := XWindow(valeurAff);
      if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
         traceMotif(xi, yi, motif);
      canvas.pen.Width := 1;
   end;
end;{ TracePoint }

procedure TgrapheGTS.TraceReticule(voie: integer);
var
   xi, yi: integer;
begin with voieOrphy[voie] do begin
      xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].valeurAff);
      yi := XWindow(valeurAff);
      if (xi <> IntSansSignif) and (yi <> IntSansSignif) then begin
         canvas.pen.color := couleur;
         canvas.moveTo(xi - scaleCurseur, yi);
         canvas.lineTo(xi + scaleCurseur, yi);
         canvas.moveTo(xi, yi - scaleCurseur);
         canvas.lineTo(xi, yi + scaleCurseur);
      end;
   end;
end;{ TraceReticule }

procedure TgrapheGts.SetHauteurFonte(H: integer);
begin
   if H <= 13 then
      H := 13 { MS sans serif 8 points = 13 pixels }
   else if H <= 16 then
      H := 16 { MS sans serif 10 points = 16 pixels }
   else
      H := 20; { MS sans serif 12 points = 20 pixels }
   canvas.font.Height := H;
   HautCarac := H;
   LargCarac := 2 * H div 3;
   Marge := largCarac div 2;
end;

procedure TgrapheGTS.Draw;

   procedure ChercheMonde;
   var
      voie: indiceVoie;
      N, Nc, hauteurCourbe: integer;
   begin { ChercheMonde }
      if AnalyseurLogique then
         N := NbreCourbe
      else
         N := 1;
      with limiteFenetre do begin
         avecAxeY := ((bottom - top) div 5 > NbreGradMin * hautCarac * N div
            4) and (AnalyseurLogique or AvecUnAxe);
         avecAxeX := (right - left) > NbreGradMin * 4 * largCarac;
         grandAxeY := (bottom - top) div 5 > NbreGraduations * HautCarac * N div 4;
         grandAxeX := (right - left) > NbreGraduations * 4 * largCarac;
         MaxGr := 2 * right;
         MinGr := -right;
      end;
      limiteCourbe := limiteFenetre;
      Inc(limiteCourbe.left, Marge);
      Dec(limiteCourbe.bottom, Marge);
      Inc(limiteCourbe.top, Marge);
      if avecAxeY then
         Inc(limiteCourbe.left, LongNombreAxe * largCarac);
      if avecAxeY and (NbreCourbe > 1) then
         Dec(limiteCourbe.right, LongNombreAxe * largCarac);
      if avecAxeX then
         Dec(limiteCourbe.bottom, hautCarac);
      for voie := voieMin to voieMax do
         with voieOrphy[voie] do begin
            CourbeTop := LimiteCourbe.top;
            CourbeBottom := LimiteCourbe.bottom;
         end;
      if AnalyseurLogique then begin
         hauteurCourbe := (LimiteCourbe.bottom - LimiteCourbe.top) div NbreCourbe - marge;
         Nc := 0;
         for voie := voieMin to voieMax do
            with voieOrphy[voie] do
               if active and (voie <> voieAbscisse) then  begin
                  CourbeTop := LimiteCourbe.top + Nc * hauteurCourbe + succ(Nc) * marge;
                  CourbeBottom := CourbeTop + hauteurCourbe;
                  Inc(Nc);
               end;
      end;
      with voieOrphy[voieAbscisse] do begin
         A := (limiteCourbe.right - limiteCourbe.left) / (Maxi - Mini);
         B := limiteCourbe.left - Mini * A;
         setDelta(grandAxeX);
      end;
      for voie := voieMin to voieMax do
         with voieOrphy[voie] do
            if active and (voie <> voieAbscisse) then begin
               A := (CourbeTop - CourbeBottom) / (Maxi - Mini);
               B := CourbeBottom - Mini * A;
               setDelta(grandAxeY);
            end;
   end; { ChercheMonde }

{$I axes}

begin
   with limiteFenetre do
      setHauteurFonte((bottom - top) div 30);
   canvas.pen.Width := 1;
   canvas.pen.style := psDot;
   canvas.pen.mode  := pmCopy;
   canvas.pen.color := voieOrphy[voieT].couleur;
   canvas.brush.style := bsClear;
   setBkMode(canvas.handle, transparent);
   chercheMonde;
   drawAxis;
   TraceCourbes;
   TraceLigneSynchro;
   ReticuleTrace := False;
end;

procedure TgrapheGTS.TraceCourbes;

   procedure TraceUneCourbe(voie: integer);

      procedure TracePoints;
      var
         i: integer;
         xi, yi: integer;
      begin with voieOrphy[voie] do begin
            canvas.brush.color := couleur;
            if (modeAcq in [AcqClavier, AcqPoint]) and (NbreAff < 32) then
               canvas.pen.Width := 3;
            for i := 0 to pred(NbreAff) do begin
               xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
                  TraceMotif(xi, yi, motif);
            end;{i}
            canvas.pen.Width := 1;
         end;
      end;{tracePoints}

      procedure traceLignes;
      var
         i: integer;
         PointPrecedent: boolean;
         xi, yi: integer;
      begin with voieOrphy[voie] do begin
            PointPrecedent := False;
            for i := 0 to pred(NbreAff) do begin
               xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi = IntSansSignif) or (yi = IntSansSignif) then
                  pointPrecedent := False
               else if PointPrecedent then
                  canvas.LineTo(xi, yi)
               else
               begin
                  canvas.MoveTo(xi, yi);
                  pointPrecedent := True;
               end;
            end;{for i}
         end; {i}
      end;{traceLignes}

   begin with voieOrphy[voie] do begin
         if NbreAff = 0 then
            exit;
         canvas.pen.style := psSolid;
         canvas.pen.color := couleur;
         if oTraceLigne then
            TraceLignes;
         if oTracePoints then
            TracePoints;
      end;
   end;{ TraceUneCourbe }

var
   voie: indiceVoie;
begin
   canvas.pen.Width := 1;
   for voie := succ(voieMin) to voieMax do
      if voieOrphy[voie].active and (voie <> voieAbscisse) then
         traceUneCourbe(voie);
end; // traceCourbes


procedure TgrapheGTS.TracePoints(jdebut,jfin : integer);

   procedure TraceUneCourbe(voie: integer);

      procedure TracePoints;
      var
         i: integer;
         xi, yi: integer;
      begin
         with voieOrphy[voie] do begin
            canvas.brush.color := couleur;
            if (modeAcq in [AcqClavier, AcqPoint]) and (NbreAff < 32) then
               canvas.pen.Width := 3;
            for i := jdebut to jfin do begin
               xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
                  TraceMotif(xi, yi, motif);
            end;{i}
            canvas.pen.Width := 1;
         end;
      end;{tracePoints}

      procedure traceLignes;
      var
         i: integer;
         PointPrecedent: boolean;
         xi, yi: integer;
      begin
         with voieOrphy[voie] do begin
            PointPrecedent := False;
            if jdebut>0 then dec(jdebut);
            for i := jdebut to jfin do begin
               xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi = IntSansSignif) or (yi = IntSansSignif) then
                  pointPrecedent := False
               else if PointPrecedent then
                  canvas.LineTo(xi, yi)
               else begin
                  canvas.MoveTo(xi, yi);
                  pointPrecedent := True;
               end;
            end;{for i}
         end; {i}
      end;{traceLignes}

   begin
      with voieOrphy[voie] do begin
         if NbreAff = 0 then
            exit;
         canvas.pen.style := psSolid;
         canvas.pen.color := couleur;
         if oTraceLigne then
            TraceLignes;
         if oTracePoints then
            TracePoints;
      end;
   end;{ TraceUneCourbe }

var
   voie: indiceVoie;
begin
   canvas.pen.Width := 1;
   for voie := succ(voieMin) to voieMax do
      if voieOrphy[voie].active and (voie <> voieAbscisse) then
         traceUneCourbe(voie);
end; // tracePoints

destructor TgrapheGTS.Destroy;
var
   i: indiceVoie;
begin
   for i := voieMin to VoieMax do
      voieOrphy[i].Free;
   inherited Destroy;
end;

constructor TgrapheGTS.Create;

   procedure ValeurDefaut;
   begin
      voieAbscisse := voieA;
      oTraceLigne := True;
      oTracePoints := False;
      quadrillage := False;
      AnalyseurLogique := False;
      AvecUnAxe := True;
      modeAcq := AcqTemps;
      deltaBouge := 2;
   end;

var
   voie: indiceVoie;
begin
   inherited Create;
   tension := Tunite.Create;
   tension.nomUnite := 'V';
   BaseDeTemps := TbaseDeTemps.Create;
   voieOrphy[voieT] := BaseDeTemps;
   EntreeBinaire := TentreeBinaire.Create;
   voieOrphy[voieEB] := EntreeBinaire;
   Magnum  := Tmagnum.Create;
   voieOrphy[voieMagnum] := Magnum;
   SortieAnalogique := TsortieAnalogique.Create;
   Frequence := TvoieNonEA.Create('Fech', 'Hz', 'frquence', -1);
   (*  voieOrphy[voieF] := Frequence;*)
   voieOrphy[voieSA] := SortieAnalogique;
   EntreeClavier := TentreeClavier.Create;
   voieOrphy[voieClavier] := EntreeClavier;
   for voie := voieA to voieAnaMax do
   begin
      voieOrphy[voie] := Tcourbe.Create('V' + IntToStr(voie - voieA + 1), 'V', '', voie);
      voieOrphy[voie].setMinMax;
   end;
   voieOrphy[voieA].active := True;
   voieOrphy[succ(voieA)].active := True;
   for voie := voieMin to voieMax do
      with voieOrphy[voie] do
         if couleur = clWindow then
            couleur := clActiveCaption;
   ValeurDefaut;
end;

constructor Tcourbe.Create(Anom: string; Aunite: string;
   Asignif: string; Anumero: integer);
begin
   inherited Create(Anumero);
   active := False;
   A := 1;
   B := 0;
   nom := Anom;
   NomUnite := Aunite;
   signif := Asignif;
   axeLog := False;
end;

constructor TsortieAnalogique.Create;
begin
   inherited Create('Vsa', 'V', trTensionSA, voieSA);
   DebutSortie := 0;
   FinSortie := maxiHardSA;
   mini := 0;
   maxi := 5;
   deltatSA := 1000;
   numOrphy := -1;
   inverse := False;
end;

constructor TentreeClavier.Create;
begin
   inherited Create('X', '', '', voieClavier);
end;

constructor TbaseDeTemps.Create;
begin
   inherited Create('t', 's', 'dure', voieT);
   numOrphy := -1;
end;

destructor Tcourbe.Destroy;
begin
   inherited Destroy;
end;

procedure Tcourbe.SetDelta(grandAxe: boolean);
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const
   coeff: array[0..5] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
   AbsMax, AbsMin, largeur: double;
   Exponent, index: integer;
   N: byte;
begin
   AbsMax := abs(maxi);
   AbsMin := abs(mini);
   if AbsMax < AbsMin then
      AbsMax := AbsMin;
   axeDateTime := (nomUnite = 's') and (AbsMax > QuartdHeure);
   if axeDateTime then
   begin
      NbreChiffre := longNombreAxe;
      DeltaAxe := (Maxi - Mini) / 6;
      if DeltaAxe > UnJour then
         DeltaAxe := ceil(DeltaAxe / UnJour) * UnJour
      else if DeltaAxe > UneHeure then begin
         N := round(DeltaAxe / UneHeure);
         case N of
            1, 2, 3, 4: ;
            5: N := 6;
            6: ;
            7: N := 8;
            8: ;
            9, 10, 11: N := 12;
            12: ;
            13, 14, 15, 16, 17: N := 12;
            18, 19, 20, 21, 22, 23, 24: N := 24;
         end;
         DeltaAxe := N * UneHeure;
      end
      else if DeltaAxe > QuartdHeure then
         DeltaAxe := ceil(DeltaAxe / QuartdHeure) * QuartdHeure
      else
         DeltaAxe := ceil(DeltaAxe / UneMinute) * UneMinute;
      PointDebut := 0;
      Exposant := 1;
      exit;
   end;
   Exponent := floor(Log10(Maxi - Mini));
   largeur := (maxi - mini) * dix(-exponent);
   index := 0;
   if grandAxe then
      while ((largeur / coeff[index]) >= NbreGraduations) and (index < 5) do
         Inc(index)
   else
      while ((largeur / coeff[index]) >= NbreGradMin) and (index < 5) do
         Inc(index);
   DeltaAxe := coeff[index] * dix(exponent);
   NbreChiffre := ceil(Log10(AbsMax)) - floor(Log10(DeltaAxe));
   PointDebut := ceil(mini / deltaAxe) * deltaAxe;
   Index := Exponent div 3;
   if ((Exponent mod 3) < 0) then
      Dec(index);
   { la division euclidienne des informaticiens <> div des maths ! }
   Exposant := Dix(3 * Index);
   N := ceil(Log10(AbsMax / Exposant));
   if N > NbreChiffre then
      NbreChiffre := N;
   if NbreChiffre < 3 then
      NbreChiffre := 3;
   if (NbreChiffre > longNombreAxe) or axeLog then
      NbreChiffre := longNombreAxe;
end;

function Tcourbe.FormatAxe(x: double): string;
begin
   FormatAxe := FloatToStrF(x, ffGeneral, NbreChiffre, 0);
end;

function TsortieAnalogique.ValeurVolt(V: integer): double;
begin
   Result := V * 5.0 / maxiHardSA;
end;


function TsortieAnalogique.ValeurReal(V: integer): double;
begin
   Result := valeurVolt(V);
end;

function TsortieAnalogique.GetNomVoieOrphy: string;
begin
   Result := 'SA';
end;

function TentreeClavier.GetNomVoieOrphy: string;
begin
   Result := 'Clavier';
end;

function TentreeBinaire.GetNomVoieOrphy: string;
begin
   Result := 'EB';
end;

procedure TsortieAnalogique.CalcVecteur;
var
   i, N: integer;
begin
   N := finSortie - debutSortie;
   for i := 0 to N do begin
      if inverse then
         vecteurCourant[i] := valeurReal(finSortie - i)
      else
         vecteurCourant[i] := valeurReal(i + debutSortie);
      vecteurAff[i] := vecteurCourant[i];
   end;
end;

procedure TsortieAnalogique.Load(Avoie: string; fichier: tMemIniFile);
begin
   inherited load(Avoie, fichier);
   with fichier do
   begin
      DebutSortie := ReadInteger(Avoie, 'Debut', 50);
      FinSortie := ReadInteger(Avoie, 'Fin', 250);
      DeltatSA := ReadInteger(Avoie, 'Deltat', DeltatSA);
      inverse := ReadBool(Avoie, 'Inverse', False);
   end;
   CalcVecteur;
end;

procedure TvoieNonEA.setMinMax;
var
   maxmax: double;
   exponent: integer;
begin
   if Mini >= Maxi then
      Maxi := Mini + 1;
   MaxMax := abs(maxi);
   if MaxMax < abs(mini) then
      MaxMax := abs(mini);
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0 then
      Exponent := (Exponent div 3) - 1
   else
      Exponent := Exponent div 3;
   { la division euclidienne des informaticiens <> div des maths ! }
   ExposantAff := Dix(3 * Exponent);
   NbreAvantVirgule := floor(Log10(MaxMax / ExposantAff)) + 2;
end;

procedure TsortieAnalogique.Store(Avoie: string; fichier: tMemIniFile);
begin
   inherited store(Avoie, fichier);
   with fichier do
   begin
      WriteInteger(Avoie, 'Debut', DebutSortie);
      WriteInteger(Avoie, 'Fin', FinSortie);
      WriteInteger(Avoie, 'Deltat', DeltatSA);
      WriteBool(Avoie, 'Inverse', inverse);
   end;
end;

procedure TbaseDeTemps.Load(Avoie: string; fichier: tMemIniFile);
begin
   inherited load(Avoie, fichier);
   Maxi := fichier.ReadFloat(Avoie, 'Dure', 1.0);
end;

procedure TbaseDeTemps.Store(Avoie: string; fichier: tMemIniFile);
begin
   inherited store(Avoie, fichier);
   with fichier do
      WriteFloat(Avoie, 'Dure', Maxi);
end;

function TsortieAnalogique.CalibreVoie: string;
begin
   Result := '0-5 V';
end;


constructor TentreeBinaire.Create;
begin
   inherited Create('EB', '', trEntreeBin, voieEB);
   mini := 0;
   nbreEB := 3;
end;

function TentreeBinaire.ValeurReal(V: integer): double;
begin
   Result := V and masque;
end;

procedure TentreeBinaire.Load(Avoie: string; fichier: tMemIniFile);
begin
   inherited load(Avoie, fichier);
   NbreEB := fichier.ReadInteger(Avoie, 'Nbre', 8);
   Mini := 0;
end;

procedure TentreeBinaire.setNbreEB(Value: byte);
var
   i: byte;
   j: word;
begin
   FnbreEB := Value;
   Masque := 0;
   j := 1;
   for i := 1 to FnbreEB do begin
      Masque := Masque + j;
      j := j * 2;
   end;
   Maxi := Masque + 1;
end;

procedure TentreeBinaire.Store(Avoie: string; fichier: tMemIniFile);
begin
   inherited store(Avoie, fichier);
   fichier.WriteInteger(Avoie, 'Nbre', FnbreEB);
end;

function TentreeBinaire.CalibreVoie: string;
begin
   Result := IntToStr(masque);
end;

function TentreeBinaire.NomValeurUnite(Nombre: double): string;
var
   entier: word;
   hexa: string;
begin
   entier := round(nombre);
   hexa := HexW(entier);
   Delete(Hexa, 1, 2);
   NomValeurUnite := IntToStr(entier) + '=' + Hexa + 'h=' +
      copy(BinaryW(entier), 17 - FnbreEB, FnbreEB) + 'b';
end;

procedure TgrapheGTS.setNbreInter(Avalue: integer);

   procedure getNbrePea;
   var
      OKpea: boolean;
      voie:  indiceVoie;
   begin
      case NbreVoie of
         1: NbrePea := 1;
         2: NbrePea := 2;
         3, 4: begin
            OKpea := True;
            for voie := voieA to pred(voieA + NbreVoie) do
               OKpea := OKpea and (voieOrphy[voie].numOrphy < 4);
            if OKpea then
               NbrePea := 4
            else begin
               OKpea := True;
               for voie := voieA to pred(VoieA + NbreVoie) do
                  OKpea := OKpea and (voieOrphy[voie].numOrphy > 3);
               if OKpea then
                  NbrePea := 4
               else
                  NbrePea := 8;
            end;
         end;
         else
            NbrePea := 8;
      end;{case}
   end;

var
   voie, v: indiceVoie;
   Nmax: integer;
   voieTampon: Tcourbe;
begin
   FnbreInter := Avalue;
   for voie := voieA to voieMax do
      with voieOrphy[voie] do
         if GBvoie <> nil then
         begin
            GBvoie.tag := voie;
            Labelvoie.tag := voie;
         end;
   if modeAcq = AcqMAgnum then
   begin
      NbreVoie := 1;
      exit;
   end;
   NbreVoie := 0;
   for voie := voieA to voieAnaMax do
      if voieOrphy[voie].active then begin
         Inc(NbreVoie);
         for v := voieA to pred(voie) do
            if not (voieOrphy[v].active) then begin
               voieTampon := voieOrphy[v];
               voieOrphy[v] := voieOrphy[voie];
               voieOrphy[voie] := voieTampon;
               break;
            end;
      end;
   if NbreVoie = 0 then
   begin
      voieOrphy[voieA].active := True;
      NbreVoie := 1;
   end;
   getNbrePea;
   Nmax := MaxVecteur div NbrePea;
   if Avalue > NMax then
      Avalue := NMax;
   if Avalue < MinPointsTemps then
      Avalue := MinPointsTemps;
   FnbreInter := Avalue;
end; { setNbreInter }


procedure TGrapheGTS.TraceLigneSynchro;
begin
   if (modeSynchro = s_seuil) and (voieAbscisse <> voieSynchro) and
      (modeAcq = AcqTemps) then
      with canvas do begin
         pen.style := psDash;
         pen.mode  := pmNotXOR;
         pen.color := voieOrphy[voieSynchro].couleur;
         with voieOrphy[voieSynchro] do
            yseuil := CourbeBottom + round((CourbeTop - CourbeBottom) * seuilSynchro / 255);
         moveTo(limiteCourbe.left, yseuil);
         lineTo(limiteCourbe.right, yseuil);
         canvas.TextOut(LimiteCourbe.left, yseuil - 10, Fleche[synchroUp]);
         pen.style := psSolid;
      end;
end;

{$I Gauss}

constructor Tmagnum.Create;
begin
   inherited Create('x', 'm', trMagnum, voieMagnum);
   nomUnite := 'm';
   modeAngulaire := False;
   modeRadian := False;
   TraitMagnumE := 0.0018;
   increment := traitMagnum;
   numOrphy := -1;
   NbreMax := 256;
end;

procedure Tmagnum.RecopieMagnum;
var
   vW, maxW, trait: double;
   k: integer;
   z, maxz, minz, zeroz: integer;
begin
   if isChute then begin
      if Negatif then begin
         trait := -2 * traitMagnumE;
         maxi  := 0;
         mini  := (NbreMagnum + 2) * trait;
      end
      else begin
         trait := +2 * traitMagnumE;
         maxi  := (NbreMagnum + 2) * trait;
         mini  := 0;
      end;
      for k := 0 to pred(NbreMagnum) do
         vecteurAff[k] := k * trait;
   end
   else begin
      z := vecteurLu[0];
      maxz := z;
      minz := z;
      for k := 1 to pred(NbreMagnum) do
      begin
         z := vecteurLu[k];
         if z > maxz then
            maxz := z;
         if z < minz then
            minz := z;
      end;
      zeroz := (maxz + minz) div 2;
      maxW  := 0;
      for k := 0 to pred(NbreMagnum) do begin
         vecteurAff[k] := (vecteurLu[k] - zeroz) * traitMagnumE;
         vW := vecteurAff[k];
         if abs(vW) > maxW then
            maxW := abs(vW);
      end;
      if maxW > 10 * TraitMagnumE then
      begin
         maxi := maxW * 1.05;
         mini := -maxi;
      end
      else
      begin
         maxi := 11 * traitMagnumE;
         mini := -maxi;
      end;
   end;
end;

procedure Tmagnum.TestChute;

   procedure VerifChute;
   var
      k, i: integer;
      dt1, dt2, debutChuteT: extended;
   begin with grapheGTS, magnum do begin
         if NbreMagnum < 3 then begin
            NbreMagnum := 1;
            NbreInter  := NbreMagnum;
            exit;
         end;
         Dec(NbreMagnum, 1); { support  la fin }
         NbreInter := NbreMagnum;
         with VoieOrphy[voieT] do begin
            dt1 := vecteurAff[1] - vecteurAff[0];
            dt2 := vecteurAff[2] - vecteurAff[1];
         end;
         if dt1 > dt2 then begin
            debutChuteT := VoieOrphy[voieT].vecteurAff[1];
            Dec(NbreMagnum, 1); { support au dbut }
            NbreInter := NbreMagnum;
            for k := 0 to pred(NbreInter) do begin
               VoieOrphy[voieMagnum].vecteurAff[k] := VoieOrphy[voieMagnum].vecteurAff[k + 1];
               VoieOrphy[voieT].vecteurAff[k] :=
                  VoieOrphy[voieT].vecteurAff[k + 1] - debutChuteT;
            end;
         end;
         k := 0;
         repeat
            Inc(k);
            dt1 := VoieOrphy[voieT].vecteurAff[k] - VoieOrphy[voieT].vecteurAff[k - 1];
         until (dt1 > 0.0001) or (k > NbreInter); // 100 s mini sinon vibrations
         if k > 1 then
         begin
            k := k - 1;
            debutChuteT := VoieOrphy[voieT].vecteurAff[k];
            Dec(NbreMagnum, k);
            NbreInter := NbreMagnum;
            for i := 0 to pred(NbreInter) do
            begin
               VoieOrphy[voieMagnum].vecteurAff[i] := VoieOrphy[voieMagnum].vecteurAff[i + k];
               VoieOrphy[voieT].vecteurAff[i] :=
                  VoieOrphy[voieT].vecteurAff[i + k] - debutChuteT;
            end;
         end;
         if NbreInter < 1 then
            NbreInter := 1;
      end;
   end; // StopChute

var
   k: integer;
   Nplus, Nmoins, Nmin: integer;
begin
   IsChute := NbreMagnum > 16;
   if IsChute then begin
      Nplus := 0;
      Nmoins := 0;
      Nmin := NbreMagnum div 3;
      for k := 2 to (NbreMagnum - 2) do
         if vecteurLu[k] > vecteurLu[pred(k)] then
            Inc(Nplus)
         else
            Inc(Nmoins);
      IsChute := (Nplus < Nmin) or (Nmoins < Nmin);
   end;
   if isChute then
      verifChute;
end; // TestChute

function Tmagnum.ValeurReal(V: integer): double;
begin
   Result := V * traitMagnumE;
end;

procedure Tmagnum.Load(Avoie: string; fichier: tMemIniFile);
begin
   inherited load(Avoie, fichier);
   with fichier do
   begin
      modeAngulaire := ReadBool(Avoie, 'angle', False);
      modeRadian := ReadBool(Avoie, 'radian', False);
      increment := ReadFloat(Avoie, 'increment', TraitMagnum);
      nbreMax := ReadInteger(Avoie, 'nbremax', 256);
      traitMagnumE := ReadFloat(Avoie, 'taille', TraitMagnum);
      Negatif := ReadBool(Avoie, 'Negatif', True);
   end;
end;

procedure Tmagnum.Store(Avoie: string; fichier: tMemIniFile);
begin
   inherited store(Avoie, fichier);
   with fichier do begin
      WriteBool(Avoie, 'radian', FModeRadian);
      WriteBool(Avoie, 'angle', FModeAngulaire);
      WriteBool(Avoie, 'Negatif', Negatif);
      WriteInteger(Avoie, 'nbremax', NbreMax);
      WriteFloat(Avoie, 'Taille', traitMagnumE);
      WriteFloat(Avoie, 'Increment', increment);
   end;
end;

procedure Tmagnum.setModeAngulaire(Value: boolean);
begin
   if FmodeAngulaire <> Value then
   begin
      FmodeAngulaire := Value;
      if Value then begin
         if modeRadian then
            nomUnite := 'rad'
         else
            nomUnite := '';
      end
      else begin
         TraitMagnumE := traitMagnum;
         nomUnite := 'm';
      end;
   end;
end;

procedure Tmagnum.setModeRadian(Value: boolean);
begin
   if (FmodeRadian <> Value) and FmodeAngulaire then begin
      FmodeRadian := Value;
      if Value then
         nomUnite := 'rad'
      else
         nomUnite := '';
   end;
end;

function Tmagnum.AffVoltmetre: string;
begin
   Result := '';
end;

function Tmagnum.CalibreVoie: string;
begin
   Result := '';
end;

function Tmagnum.Date: double;
var
   posb: integer;
   strDate: string;
   compteur: longint;
begin
   posB := pos(' ', reponse);
   strDate := copy(reponse, succ(posB), length(reponse) - posB);
   try
      VecteurLu[NbreMagnum] := StrToInt(copy(reponse, 1, pred(posB))) - zeroMagnum;
   except
      VecteurLu[NbreMagnum] := 0;
   end;
   posB := pos(' ', strDate);
   try
      compteur := StrToInt(copy(strDate, 1, pred(posB))) * 65536 +
         StrToInt(copy(strDate, succ(posB), length(strDate) - posB));
   except
      compteur := 0;
   end;
   if nbreMagnum = 0 then
      compteurDebut := compteur;
   Result := (compteur - compteurDebut) * deltatMagnum;
end;

end.

