unit GTSDEF;

interface

uses classes, sysutils, oomisc, inifiles, graphics,
     uniteker, maths, gtsutil, adport, constgts, registry,
     math;

Const
{ MaxVecteur = 6400 }
{ maxi possible : 200 blocs*32 octets=6400 }
{ mode PEAi /8 voies = 800 }
{ mode PEA0 /10 (Npea0Max) = 640 }

    MasseFlottante : boolean = true;
    premiereFois : boolean = true;
    MajTimerAttente = 110;
    MajTimerMagnum = 550;  
    MajTimerTemps = 220;
    voieT = 1;
    voieSA = 2;  {sortie analogique}
    voieClavier = 3;
    voieMagnum = 9;
    voieEB = 4; { entres binaires }
    voieA = 5;
    voieB = 6;
    voieC = 7;
    voieD = 8;
    voieAnaMax = voieD;
    voieMin = voieT;
    VoieMax = voieMagnum;
    MaxiHardSA = 255;
    MaxiHardEA = 255;
    MinPointsTemps = 8;
    MinPointsFichier = 1;
    NomMasse : array[Boolean] of string[7] = ('Ref OV','Ref Alt');

    deltatMin2 = 0.001;{ Mini avec PEA2 et c }
    deltatMin1 = 0.0005;{ Mini avec PEA1 }
    deltatOrphy = 250E-6;
    deltatMin0 = 52.06E-6; { Mini avec PEA0 }
    dureeMax0 = 0.33; { Maxi avec PEA0 }
    LongueurBloc = 32;
    NombreBlocMax = 199;
    Npea0Max = 10; { deltatMin1/deltatMin0 }
(*    MaxiFrequence : real = 4E4;*)
    TraitMagnum = 0.0018;  { demi pas }
    ZeroMagnum = 30000;
    DeltatMagnum = 1.08506E-6;

type
    Tpea = (pea0,pea1,pea2,pea40,pea41,pea8);
    Tmotif = (mCroix,mCroixDiag,mCercle,mCarre,mLosange);
    TModeAcq = (AcqClavier,AcqTemps,AcqPoint,AcqSortieAna,AcqMagnum);
    TetatAcq = (eaAttenteSynchro,eaAttenteResul,eaTerminee);
    indiceVoie = voieT..VoieMax;
    indiceVoieHard = 0..7;
    TModeSynchro = (S_front,S_relaxe,S_clavier,s_seuil,s_bouge);
    EserieError = class(Exception);
    EgtsError = class(Exception);

Const
    DeltatMax = 16; // 16 s
    DureeMax = deltatMax*MaxVecteur; // 28 heures 
    DeltatDos = 1/18;
    PriseVoie : array[0..7] of char = 'AADDBBCC';
    LongueurUnite = 15;
    nomModeAcq : array[TModeAcq] of String =
        ('Clavier','Temps','Point par point','Sortie analog.','Magnum');
    nomModeSynchro : array[TModeSynchro] of String =
        ('Front','Relax','Clavier','Seuil','Bouge');

type
   chaineUniteCapteur = string[8];
   TCapteur = class
        AvecPotarC : boolean;
        AvecAmpliC : boolean;
        AvecAlimC : boolean;
        OffsetC : double;
        UnVoltC : double;
        nomC : string;
        nomUniteC : string;
        signifC,aideC : string;
        numeroC : byte;
        masseFlottanteC : boolean;
        constructor create;
        procedure Assign(Acapteur : Tcapteur);
   end;

   Tvoie = class(tunite)
          private
              FnumOrphy : integer;
              procedure setNumOrphy(value : integer);
              procedure setMaxi(value : double);
              procedure setMini(value : double);
              procedure setGain6(value : boolean);
          protected
              Fmini,Fmaxi : double;
              Fpente : double;
              Fgain6 : boolean;
              Function GetNomVoieOrphy : string;virtual;
          public
          Offset,UnVolt : double;
          ValeurCourante,valeurAff : double;
          AvecPotar,AvecAmpli,AvecAlim : boolean;
          signif,aide : string;
          exposantAff : double;
          nbreAvantVirgule : integer;
          vecteurAff,vecteurCourant : vecteur;
          active : boolean;
          couleur : Tcolor;
          Motif : Tmotif;
          AvecCapteur : boolean;
          property NumOrphy : integer read FnumOrphy write setNumOrphy;
          property Maxi : double read Fmaxi write setMaxi;
          property Mini : double read Fmini write setMini;
          property Gain6 : boolean read Fgain6 write setGain6;
          property NomVoieOrphy : string read GetNomVoieOrphy;
          constructor create(num : byte);
          Function affVoltmetre : string;virtual;
          Function valeurReal(V : integer) : double;virtual;
          Function valeurVolt(V : integer) : double;virtual;
          procedure RazVecteur;
          procedure setMinMax;virtual;
          procedure load(Avoie : string;Fichier : tMemIniFile);virtual;
          procedure store(Avoie : string;Fichier : tMemIniFile);virtual;
          Function calibreVoie : string;virtual;
    end;

const
   crLF = #10;
   crCR = #13;

type
  TlisteCapteur = class(TList)
  protected
    function Get(Index : Integer) : Tcapteur;
    procedure Put(Index : Integer;Item : Tcapteur);
  public
    function Add(Item : Tcapteur) : Integer;
    function IndexOf(Item : Tcapteur) : Integer;
    function Remove(Item : Tcapteur) : Integer;
    property Items[Index : Integer] : Tcapteur read Get write Put;
    Procedure Clear;override;
  end;

var
    NomFichierCapteurs : string;
    ListeCapteur : TlisteCapteur;
    Npea0 : byte; { passage de 52s  deltat=Npea0*52s }

Procedure ChargeListeCapteurs;
Procedure SauveListeCapteurs;

implementation

const
    CouleurDefaut : array[indiceVoie] of Tcolor =
       (clBlack,clBlack,clRed,clBlack,clRed,clNavy,
        clGreen,clBlack,clBlue);

Procedure SauveListeCapteurs;
{  sauvegarde de la liste  }
var fichier : tMemIniFile;
    ligne : string;
    i : integer;
    regIni : TregIniFile;
begin
    fichier := tMemIniFile.create(nomFichierCapteurs);
    try
    fichier.writeInteger('Capteurs','Nombre',ListeCapteur.count);
    for i := 0 to pred(ListeCapteur.count) do
       with ListeCapteur.items[i] do begin
        ligne := IntToStr(i+1);
        fichier.writeBool(ligne,'Etalonnage',AvecPotarC);
        fichier.writeBool(ligne,'Ampli',AvecAmpliC);
        fichier.writeBool(ligne,'Alim',AvecAlimC);        
        fichier.writeFloat(ligne,'0V',Offsetc);
        fichier.writeFloat(ligne,'1V',UnVoltc);
        fichier.writeString(ligne,'Nom',nomC);
        fichier.writeString(ligne,'Unite',nomUniteC);
        fichier.writeString(ligne,'Signif',signifC);
        fichier.writeString(ligne,'Aide',aideC);
        fichier.writeInteger(ligne,'Numero',numeroC);
        fichier.writeBool(ligne,'Masse',masseFlottanteC);        
    end;
    fichier.UpdateFile;
    fichier.free;
    except
    end;
    try
    RegIni := TRegIniFile.create(RegistreGTS);
    RegIni.writeString('Capteurs','Fichier',NomFichierCapteurs);
    RegIni.free;
    except
    end;
end;

Procedure ChargeListeCapteurs;
{  chargement de la liste si vide cration  l'aide de defaut }
var fichier : tMemIniFile;
    ligne : string;
    i : integer;
    NbreCapteurs : integer;
    CapteurCourant : Tcapteur;
begin
    fichier := tMemIniFile.create(nomFichierCapteurs);
    listeCapteur.clear;
    NbreCapteurs := fichier.readInteger('Capteurs','Nombre',0);
    for i := 1 to NbreCapteurs do begin
      CapteurCourant := Tcapteur.create;
      ListeCapteur.add(CapteurCourant);
      with CapteurCourant do begin
        ligne := IntToStr(i);
        AvecPotarC := fichier.readBool(ligne,'Etalonnage',false);
        AvecAmpliC := fichier.readBool(ligne,'Ampli',false);
        AvecAlimC := fichier.readBool(ligne,'Alim',false);        
        Offsetc := fichier.ReadFloat(ligne,'0V',0);
        UnVoltc := fichier.ReadFloat(ligne,'1V',1);
        nomC := fichier.readString(ligne,'Nom',nomC);
        nomUniteC := fichier.readString(ligne,'Unite','');
        signifC := fichier.readString(ligne,'Signif','');
        aideC := fichier.readString(ligne,'Aide','');        
        numeroC := fichier.readInteger(ligne,'Numero',0);
        masseFlottanteC := fichier.readBool(ligne,'Masse',true);        
    end end;
    if NbreCapteurs=0 then begin
       CapteurCourant := TCapteur.create;
       ListeCapteur.add(CapteurCourant);
    end;
    fichier.free;
end; { ChargeListeCapteurs }

procedure Tvoie.setNumOrphy(value : integer);
var Fval255,Fval0 : double;
begin
      if value>7 then value := 0;
      FnumOrphy := value;
      if FnumOrphy<0 then exit;
      case FnumOrphy of
          0..3 : Fval255 := 5;
          4,5 : Fval255 := 0.1;
          6 : Fval255 := 1;
          7 : Fval255 := 2;
          else Fval255 := 5;
      end;
      if Fgain6 and (FnumOrphy<4) then Fval255 := Fval255*6;
      if masseFlottante
         then begin
           Fval255 := Fval255/2;
           Fval0 := -Fval255;
         end
         else Fval0 := 0;
      Fpente := (Fval255-Fval0)/MaxiHardEA;
      setMinMax;   
end;

Procedure Tvoie.Store(Avoie : string;Fichier : tMemIniFile);
begin with fichier do begin
      WriteString(Avoie,'Nom',nom);
      WriteString(Avoie,'Unite',nomUnite);
      WriteString(Avoie,'Signif',signif);
      WriteFloat(Avoie,'Mini',Fmini);
      WriteFloat(Avoie,'Maxi',Fmaxi);
      if not avecCapteur then begin
         offset := 0;
         UnVolt := 1;
      end;
      WriteFloat(Avoie,'Offset',Offset);
      WriteFloat(Avoie,'UnVolt',UnVolt);
      WriteInteger(Avoie,'NumeroOrphy',FnumOrphy);
      WriteBool(Avoie,'Active',active);
      WriteInteger(Avoie,'Couleur',couleur);
      WriteInteger(Avoie,'Motif',ord(motif));
      WriteBool(Avoie,'Capteur',avecCapteur);
      WriteBool(Avoie,'Etalonnage',AvecPotar);
end end;

Constructor Tvoie.create(num : byte);
begin
     inherited create;
     nom := 'V'+intToStr(num);
     nomUnite := 'V';
     Fmaxi := 1;
     Fmini := 0;
     signif := '';
     Offset := 0;
     UnVolt := 1;
     ValeurCourante := 0;
     numOrphy := num-voieA;
     avecAmpli := false;
     avecAlim := false;     
     avecPotar := false;
     Motif := mCroix;
     couleur := CouleurDefaut[num];
     exposantAff := 1;
     NbreAvantVirgule := 0;
end;

Procedure Tvoie.setMinMax;
var maxmax : double;
    exponent : integer;
begin
   if NumOrphy>=0 then begin
      Fmaxi := valeurReal(MaxiHardEA);
      Fmini := valeurReal(0);
      if Fmaxi<Fmini then begin
         maxmax := Fmaxi;
         Fmaxi := Fmini;
         Fmini := maxmax;
      end;
   end;
   MaxMax := abs(Fmaxi);
   if MaxMax<abs(Fmini) then MaxMax := abs(Fmini);
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0
      then Exponent := (Exponent div 3)-1
      else Exponent := Exponent div 3;
{ la division euclidienne des informaticiens <> div des maths ! }
   ExposantAff := Dix(3*Exponent);
   NbreAvantVirgule := floor(Log10(MaxMax/ExposantAff))+2;
end; {setMinMax}

Function Tvoie.GetNomVoieOrphy : string;
begin
     if FnumOrphy>=0
        then result := 'EA'+intToStr(FnumOrphy)
        else result := ''
end;

Function Tvoie.ValeurReal(V : integer) : double;
begin
     result := valeurVolt(V);
     if avecCapteur then result := Offset+unVolt*result
end;

Function Tvoie.ValeurVolt(V : integer) : double;
begin
   result := Fpente*V;
   if masseFlottante then result := result-Fpente*127.5
end;

procedure Tvoie.RazVecteur;
var i : integer;
begin
    for i := 0 to MaxVecteur do begin
       vecteurAff[i] := 0;
       vecteurCourant[i] := 0;
    end;
end;

Procedure Tvoie.Load(Avoie : string;fichier : tMemIniFile);
begin with fichier do begin
      AvecPotar := ReadBool(Avoie,'Etalonnage',false);
      AvecCapteur := ReadBool(Avoie,'Capteur',false);
      Nom := ReadString(Avoie,'Nom',Nom);
      NomUnite := ReadString(Avoie,'Unite',NomUnite);
      Signif := ReadString(Avoie,'Signif','');
      Mini := ReadFloat(Avoie,'Mini',0);
      Maxi := ReadFloat(Avoie,'Maxi',5);
      offset := ReadFloat(Avoie,'Offset',0);
      unVolt := ReadFloat(Avoie,'UnVolt',1);
      active := ReadBool(Avoie,'Active',false);
      numOrphy := ReadInteger(Avoie,'NumeroOrphy',0);
      Couleur := readInteger(Avoie,'Couleur',couleur);
      Motif := Tmotif(readInteger(Avoie,'Motif',0));
      setMinMax;
end end;

Function Tvoie.CalibreVoie : string;
begin
   if Fmaxi*Fmini<0
      then if abs(Fmaxi)>abs(Fmini)
            then result := ''+formatUniteBrute(abs(Fmaxi))
            else result := ''+formatUniteBrute(abs(Fmini))
      else if Fmaxi>0
           then result := formatUniteBrute(Fmaxi)
           else result := formatUniteBrute(Fmini)
end;

Function Tvoie.AffVoltmetre : string;
var posVirgule,i : integer;
begin
    result := FloatToStrF(valeurAff/ExposantAff,ffFixed,3,5-NbreAvantVirgule);
    posVirgule := pos('.',result);
    for i := 0 to (NbreAvantVirgule-posVirgule) do
        result := ' '+result;
end;

function TlisteCapteur.Add(Item : Tcapteur) : Integer;
begin
     result := inherited Add(Item)
end;

function TlisteCapteur.Get(Index : Integer) : Tcapteur;
begin
  Result := Tcapteur(inherited Get(Index))
end;

function TlisteCapteur.IndexOf(Item : Tcapteur) : Integer;
begin
  Result := inherited IndexOf(Item)
end;

procedure TlisteCapteur.Put(Index : Integer;Item : Tcapteur);
begin
  inherited Put(Index,Item)
end;

function TlisteCapteur.Remove(Item : Tcapteur) : Integer;
begin
  Result := inherited Remove(Item);
  Item.free;
end;

Procedure TListeCapteur.clear;
var i : Integer;
begin
  for i := 0 to pred(Count) do Items[i].Free;
  inherited Clear;
end;

constructor Tcapteur.create;
begin
        AvecPotarC := false;
        AvecAmpliC := false;
        AvecAlimC := false;        
        Offsetc := 0;
        unVoltc := 10;
        nomC := 'T';
        NomUniteC := 'C';
        signifC := '';
        aideC := '';
end;

procedure Tcapteur.assign(Acapteur : Tcapteur);
begin
      AvecPotarC := Acapteur.avecPotarC;
      AvecAmpliC := Acapteur.avecAmpliC;
      AvecAlimC := Acapteur.avecAlimC;      
      OffsetC := Acapteur.offsetc;
      UnVoltC := Acapteur.unVoltc;
      nomC := Acapteur.nomC;
      NomUniteC := Acapteur.NomUniteC;
      signifC := Acapteur.signifC;
      aideC := Acapteur.aideC;
      masseFlottanteC := Acapteur.masseFlottanteC;      
end;

procedure Tvoie.setMaxi(value : double);
begin
   if FnumOrphy<0 then Fmaxi := value;
end;

procedure Tvoie.setMini(value : double);
begin
   if FnumOrphy<0 then Fmini := value;
end;

procedure Tvoie.setGain6(value : boolean);
begin
   if (FnumOrphy>=0) and (FnumOrphy<4) then begin
       Fgain6 := value;
       setNumorphy(FnumOrphy);
   end;
end;

initialization
    NomFichierCapteurs := changeFileExt(paramstr(0),'.CPT');
    ListeCapteur := TlisteCapteur.create;
    ChargeListeCapteurs
Finalization
    ListeCapteur.clear;
    ListeCapteur.free
end.
