const
   degre = 2;
   ordre = degre+1;

type
   matrice = array[1..ordre,1..ordre+1] of extended;
   matriceLigne = array[1..ordre] of extended;
   EsystemeLineaireError = class(Exception);
   vectorPuiss = array[0..2*degre] of extended;

Procedure ResolutionSys(A : matrice;var Result : matriceLigne);
 { Cette procdure rsoud un systme AX = B }
 { crit sous la forme AX = A(n+1) }
 { par la mthode d'limination de Gauss (pivot partiel) }

  Procedure Elimination;

    FUNCTION Pivot(i :integer):integer;
    Var n,temp : integer;
    BEGIN
      temp := i;
      for n := i+1 to ordre do
          if abs(A[n,i]) > abs(A[i,i]) then temp:= n;
      pivot:=temp;
      if abs(A[temp,i]) <= 1e-100 then
         raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
    END;

    PROCEDURE Permute(i,j:integer);
    Var temp:extended;
        k:integer;
    BEGIN
       if i<>j then For k:=i to ordre+1 do BEGIN
          temp:=A[i,k];
          A[i,k]:=A[j,k];
          A[j,k]:=temp;
       END;
    END;

  var I,J,K : integer;
      M : extended;
  begin
    for k := 1 to ordre do begin
          Permute(k,pivot(k));
          for i:=k+1 to ordre do begin
              M := A[i,k]/A[k,k];
              for j:=k+1 to ordre+1 do A[i,j]:=A[i,j]-M*A[k,j];
              A[i,k]:=0.0;
          end;
       end;
    end; {Elimination}

    Procedure Substitution;
    var I,J : integer;
        S : extended;
    begin
         try
         for I:=ordre downto 1 do begin
             S:=A[I,ordre+1];
             for J:=I+1 to ordre do S:=S-A[I,J]*Result[J];
             Result[I]:=S/A[I,I];
         end;
         except
            on EzeroDivide do raise EsystemeLineaireError.Create('Erreur rsolution systme linaire')
         end;
    end;

begin {Resolution}
  Elimination;
  Substitution;
end; {rsolution}

Function DeriveeGauss(const X,Y:vecteur;Nbre : integer;deltaX : double;var der2 : double) : double;
const NbrePointDerMin = 5;
      NbrePointDerMax = 16;
var puissX : vectorPuiss;
    A : matrice;
    Coeff : matriceLigne;
    xfin : double;

procedure initXY;
var j : integer;
    loc : extended;
    i : byte;
    puissMax : byte;
    NbrePointDer : integer;
begin
    puissMax := 2*degre;
    for i := 1 to puissMax do puissX[i] := 0;
    j := pred(Nbre);
    NbrePointDer := 1;
    repeat
          dec(j);
          inc(NbrePointDer);
    until (j=0) or
          (abs(x[j]-xfin)>deltaX) or
          (NbrePointDer=NbrePointDerMax);
    if NbrePointDer<NbrePointDerMin then
       NbrePointDer := NbrePointDerMin;
    puissX[0] := NbrePointDer;
    for i := 1 to ordre do A[i,ordre+1] := 0;
    for j := Nbre-NbrePointDer to pred(Nbre) do begin
        loc := x[j];
        for i := 1 to puissMax do begin
            puissX[i] := puissX[i]+loc;
            loc := loc*x[j];
        end;
        loc := y[j];
        for i := 1 to ordre do begin
            A[i,ordre+1] := A[i,ordre+1]+loc;
            loc := loc*x[j];
        end;
    end;
    for i := 1 to ordre do
         for j := 1 to ordre do
             A[i,j] := PuissX[i+j-2];
end;

var der : double;
    deriveeCroissante : boolean;
begin { DeriveeGauss }
    result := 0;
    der2 := 0;
    if Nbre<NbrePointDerMin then exit;
    try
    xfin := X[pred(nbre)];
    initXY;
    ResolutionSys(A,Coeff);
    der := coeff[2]+xfin*2*coeff[3];
    der2 := abs(2*coeff[3]);    
    deriveeCroissante := der*coeff[3]>0;
    if deriveeCroissante
       then result := abs(der+deltaX*2*coeff[3])
{ drive normale plus accentuation de la courbure }
       else result := abs(der);
    except
    result := 0;
    end;
end; { DeriveeGauss }




